@@ -58,7 +58,6 @@ static int last_logread=1;
 Time
 xnee_delta_time ( xnee_intercept_data * xindata)
 {
-
   if ( xindata->newtime > xindata->oldtime ) 
     {
       return ( xindata->newtime - xindata->oldtime); 
@@ -182,6 +181,28 @@ xnee_replay_synchronize (xnee_data* xd)
 }
 
 
+static void 
+xnee_replay_update_dev_ctr(xnee_data *xd, int type)
+{
+  if ( type == ButtonPress )
+    {
+      xd->button_pressed++;
+    }
+  else if ( type == ButtonRelease )
+    {
+      xd->button_pressed--;
+    }
+  else if ( type == KeyPress )
+    {
+      xd->key_pressed++;
+    }
+  else if ( type == KeyRelease )
+    {
+      xd->key_pressed--;
+    }
+  return ;
+}
+
 /**************************************************************
  *                                                            *
  * xnee_replay_main_loop                                      *
@@ -211,6 +232,9 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 
   context_display = xnee_get_display_for_recordcontext(xd);
 
+
+
+
   if ( xd->data_file == NULL)
     {
       xnee_verbose((xd, "Using stdin as file\n"));
@@ -313,6 +337,7 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 
       ret = xnee_expression_handle_session(xd, tmp, &xindata);
 
+
 /*       if ( ret == XNEE_PRIMITIVE_DATA ) { printf ("return XNEE_OK\n"); ret = XNEE_OK; } */
 
 /* 	  printf ("  ===== starting \n"); */
@@ -337,7 +362,6 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 	      xnee_verbose((xd, " <--  xnee_get_elapsed_time\n"));
 	    }
 
-
 	  if ( ret == XNEE_META_DATA )
 	    {
 	      xnee_verbose((xd, "META DATA read ... should be "
@@ -345,6 +369,8 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 	    }
 	  else if (ret!=0)
 	    {
+
+
 /* 	      printf (" CC : "); */
 	      if (xd->first_read_time==0)
               {
@@ -354,7 +380,6 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
               }
 	      
 	      
-	
 	      /* Interrupt variable set? */
 	      if (xnee_get_interrupt_action(xd))
 		{
@@ -413,34 +438,16 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 	      switch (xindata.type)
 		{
 		case XNEE_EVENT:
+		
 		  /* if type == 0, break .... BTW, why is it 0?? */
 		  if ( xindata.u.event.type == 0 ) { break ; }
+
 		  /* is it a device event ? */
                    if ( ( xindata.u.event.type >= KeyPress ) 
 		       && (xindata.u.event.type <= MotionNotify) )
 		    {
-		      if ( xindata.u.event.type == ButtonPress )
-			{
-			  xd->button_pressed++;
-			}
-		      else if ( xindata.u.event.type == ButtonRelease )
-			{
-			  xd->button_pressed--;
-			}
-		      if ( xindata.u.event.type == KeyPress )
-			{
-			  xd->key_pressed++;
-			}
-		      else if ( xindata.u.event.type == KeyRelease )
-			{
-			  xd->key_pressed--;
-			}
+		      xnee_replay_update_dev_ctr(xd, xindata.u.event.type);
 		      
-		      
-		      /*		  if ( (xd->button_pressed==0) ||
-			(xd->key_pressed==0) )
-			{
-		      */
 		      ret = xnee_replay_synchronize (xd);
 		      if (ret != XNEE_OK)
 			{
@@ -453,7 +460,8 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 		      xnee_verbose((xd," replay MAIN  new %lu   old %lu\n",xindata.newtime , xindata.oldtime ));
 		      replayable = 
 			xnee_replay_event_handler(xd, 
-						  &xindata, last_elapsed);
+						  &xindata, 
+						  last_elapsed);
 		      
 		    }
 		  else
@@ -484,6 +492,31 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 					      xindata.u.reply.type, 
 					      XNEE_REPLAYED);
 		  break;
+		  
+		case XNEE_PROTO_XINPUT_EVENT_MASTER:
+		  xnee_verbose((xd, "READ A XINPUT EVENT MASTER\n")); 
+		  break;
+		case XNEE_PROTO_XINPUT_EVENT_SLAVE:
+		  xnee_verbose((xd, "READ A XINPUT EVENT SLAVE\n")); 
+		  
+		  xnee_replay_update_dev_ctr(xd, 
+					     (xindata.u.xievent.type-xnee_get_xinput_event_base(xd->fake)));
+		  
+		  ret = xnee_replay_synchronize (xd);
+		  if (ret != XNEE_OK)
+		    {
+		      xnee_verbose((xd, "xnee_replay_main_loop return %d\n", 
+				    ret));
+		      return ret;
+		    }
+
+		  replayable = 
+		    xnee_replay_event_handler(xd, 
+					      &xindata, 
+					      last_elapsed);
+		  
+		  break;
+
 		default:
 		  xnee_verbose((xd, 
 				"xnee_replay_MainReplayLoop: Unknown type \n"));
@@ -507,10 +540,10 @@ xnee_replay_main_loop(xnee_data *xd, int read_mode)
 	    {
  	      break; 
 	    }
-
 	  ret = xnee_expression_handle_session(xd, tmp, &xindata);
 /* 	  printf ("  and again  (%d, %s)\n", ret, xnee_get_err_description(ret)); */
 	  last_logread = 0;
+
 	}
     }
 
@@ -830,6 +863,7 @@ xnee_replay_init          (xnee_data* xd)
 
   xd->first_read_time = 0;
 
+
   if ( xnee_no_rep_resolution(xd) == 0 )
   {
      int ret ; 
